import os
import struct
import time
import zlib


def crc32_of(data: bytes) -> int:
    """
    Compute CRC-32 of 'data'.
    """
    return zlib.crc32(data) & 0xffffffff


# -----------------------------------------------------------------------------
# Ping
# -----------------------------------------------------------------------------

def build_ping_request(version: int = 0) -> bytes:
    """
    Build a Ping request:
      - Nonce (32 bytes)
      - Version (1 byte)
      - CRC-32 (4 bytes)
    Total = 37 bytes
    """
    nonce = os.urandom(32)
    partial = nonce + struct.pack("!B", version)
    crc_val = crc32_of(partial)
    return partial + struct.pack("!I", crc_val)


def parse_ping_request(data: bytes):
    """
    Parse a Ping request (37 bytes).
    Return (nonce, version) or None if invalid.
    """
    if len(data) != 37:
        return None
    nonce = data[:32]
    version = data[32]
    crc_in = struct.unpack("!I", data[33:37])[0]
    partial = data[:33]
    crc_calc = crc32_of(partial)
    if crc_calc != crc_in:
        return None
    return (nonce, version)


def build_ping_response(version: int, answer_code: int) -> bytes:
    """
    Build a Ping response:
      - Timestamp (8 bytes)
      - Version (1 byte)
      - Answer code (1 byte)
      - CRC-32 (4 bytes)
    Total = 14 bytes
    """
    timestamp = struct.pack("!d", time.time())
    partial = timestamp + struct.pack("!B", version) + struct.pack("!B", answer_code)
    crc_val = crc32_of(partial)
    return partial + struct.pack("!I", crc_val)


def parse_ping_response(data: bytes):
    """
    Parse a Ping response (14 bytes).
    Return (timestamp, version, answer_code) or None if invalid.
    """
    if len(data) != 14:
        return None
    timestamp = struct.unpack("!d", data[:8])[0]
    version = data[8]
    answer_code = data[9]
    crc_in = struct.unpack("!I", data[10:14])[0]
    partial = data[:10]
    crc_calc = crc32_of(partial)
    if crc_calc != crc_in:
        return None
    return (timestamp, version, answer_code)


# -----------------------------------------------------------------------------
# Handshake
# -----------------------------------------------------------------------------

def build_handshake_message(ephemeral_pubkey: bytes, ephemeral_signature: bytes, timestamp: float) -> bytes:
    """
    Build a handshake message:
      - ephemeral_pubkey (64 bytes)
      - ephemeral_signature (72 bytes, DER + zero-pad)
      - timestamp (8 bytes)
      - CRC-32 (4 bytes)
    Total = 148 bytes
    """
    if len(ephemeral_pubkey) != 64:
        raise ValueError("ephemeral_pubkey must be 64 bytes")
    if len(ephemeral_signature) > 72:
        raise ValueError("ephemeral_signature too large")

    sig_padded = ephemeral_signature.ljust(72, b'\x00')
    ts_bytes = struct.pack("!d", timestamp)
    partial = ephemeral_pubkey + sig_padded + ts_bytes
    crc_val = crc32_of(partial)
    return partial + struct.pack("!I", crc_val)


def parse_handshake_message(data: bytes):
    """
    Parse a handshake message (148 bytes).
    Return (ephemeral_pubkey, ephemeral_signature, timestamp) or None if invalid.
    """
    if len(data) != 148:
        return None
    ephemeral_pubkey = data[:64]
    sig_padded = data[64:136]
    ts_bytes = data[136:144]
    crc_in = struct.unpack("!I", data[144:148])[0]

    partial = data[:144]
    crc_calc = crc32_of(partial)
    if crc_calc != crc_in:
        return None

    ephemeral_signature = sig_padded.rstrip(b'\x00')
    timestamp = struct.unpack("!d", ts_bytes)[0]
    return (ephemeral_pubkey, ephemeral_signature, timestamp)
